// ==================================================================
// Test.java  Moladlib test and demonstration
// (C) 2003, Michael Glickman
// ------------------------------------------------------------------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
// ==================================================================

package com.palmcrust.moladlibtest;

import javax.microedition.midlet.MIDlet;
import java.util.Vector;
import com.palmcrust.moladlib.*;

public class Test extends MIDlet
{
	static final String[] locNames =
		 {"Jerusalem", "Melbourne", "New York"};

	static final int[][] locLatitudes = 
		// Degrees, minutes, seconds, flag: 0-North, 1-South
		{ { 31, 47, 0, 0},		// Jerusalem
          { 37, 47, 0, 1},		// Melbourne (Southern Hemisphere)
		  { 40, 47, 0, 0}		// New York	
		};

	static final int[][] locLongitudes = 
		// Degrees, minutes, seconds, flag: 0-East, 1-West
		{ { 35, 13, 0, 0},		// Jerusalem
		  {144, 58, 0, 0},		// Melbourne
		  { 73, 58, 0, 1}		// New York (Western Hemisphere)
		};

	public Test()
	{
		doTest();	
	}

	public void startApp() 
	{
		notifyDestroyed();
	}

	public void pauseApp() 
	{	}

	public void destroyApp(boolean unc) 
	{	}


	//===============================================================================
	// In order to run a test for a particular secular date, specify 
	// Target-Date: ddmmyyyy  in Moladliblest.jad
	
	private void doTest()
	{
		EventData.resetDisplaySettings();

		DateHolder dh = new DateHolder();
		//DateData dd;

		String textDate = getAppProperty("Target-Date");
		
		if (textDate != null && textDate.length() > 4)
		{
			int day = Integer.parseInt(textDate.substring(0,2));
			int month = Integer.parseInt(textDate.substring(2,4))-1;
			int year = Integer.parseInt(textDate.substring(4));
			D("Day " + day + " monbth = " + month + " year = " + year);
			dh.acceptSecularDate(new DateData(day, month, year));
			D("Jad Date");
		} else {
			dh.acceptCurrentDate();
			D("Today is");
		}
		
		
		D("\tHebrew Date: " + dateToTextHebrew(dh));
		D("\tCommon Date: " + dateToTextCommon(dh));
		D("\tAbsolute Date: " + dh.dateAbsolute);

		D("");
		D("Sunrise and sunset times (disregarding DST):");
		LocationHolder lh = new LocationHolder();
		int[] tmeZone = new int[3]; // Hours, Minutes, Direction: 0 ahead UTC, 1 - behind UTC
		int sunriseTime, sunsetTime;
		
		for (int i=0; i<locNames.length; i++) {
			if (lh.acceptLatitude(locLatitudes[i]) &&
				lh.acceptLongitude(locLongitudes[i])) {

 				lh.getIntrinsicTimeZone(locLongitudes[i], tmeZone);
 				lh.acceptTimeZone(tmeZone);
				sunriseTime = lh.getSunRiseTime(dh);
				sunsetTime = lh.getSunSetTime(dh);

				D("\t" + locNames[i] + ": " + timeToText(sunriseTime) + " / " + timeToText(sunsetTime));
			}
		}

		D("");


		showCurrentEvents("Today's events", EventData.getEventNamesForDate(dh, lh, false));
		listEvents("Coming events", EventData.getListOfEvents(dh, lh, false), 5);
		listEvents("Past events", EventData.getListOfEvents(dh, lh, true), 5);


		

	}



	private void showCurrentEvents(final String header, final Vector eventNames)
	{
		int count = eventNames.size();
		if (count <= 0) return;

		
		D(header);		
		for (int i=0; i<count; i++)
			D("\t" + ((String) eventNames.elementAt(i)).substring(1));	
	
		D("");	
	}

	private void listEvents(final String header, final Vector events, int maxCount)
	{
		int count = Math.min( events.size(), maxCount);
		if (count <= 0) return;
		
		D(header);		
		EventData ed;
		String name;
		
		for (int i=0; i<count; i++) {
			ed = (EventData) events.elementAt(i);			
			// We use long event name
			name = EventData.getHolidayName(ed.eventData[0], 0) + ":  " + dateToTextCommon(ed.eventDate);	
			if (ed.isKatan) name += " Katan";
			D("\t" + name);
		}	
	
		D("");	
	}

	String dateToTextCommon(final DateHolder dh)
	{
		final DateData dd = dh.dateSecular;
		return  dd.day + " " + SecularCalendar.getMonthName(dd.month) + " " + dd.year;
	}

	String dateToTextHebrew(final DateHolder dh)
	{
		final DateData dd = dh.dateHebrew;
		return  dd.day + " " + HebCalendar.getMonthName(dd.month) + " " + dd.year;
	}

	String timeToText(int tme)
	{
		int hrs = tme / 3600;
		int min = tme % 3600;
		int sec = min % 60;
		min /= 60;

		return String.valueOf(hrs) + ':' + min + ':' + sec;
	}

	private void D(final String text)
	{
		System.out.println(text);
		System.out.flush();
	}

}